/*
 * Created on 14.11.2014
 *
 * Author:      ES
 * Copyright:   Contecon Software GmbH
 * 
 * History:     14.11.2014 created
 * 
 */
package de.contecon.picapport.plugin.pdf;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDDocumentInformation;
import org.apache.pdfbox.util.PDFImageWriter;

import com.adobe.xmp.XMPException;

import de.contecon.imageutils.CcXMPMetaData;
import de.contecon.picapport.plugins.IPicApportPlugInLogger;
import de.contecon.picapport.plugins.otherformats.IOtherFileFormat;
import de.contecon.picapport.plugins.otherformats.OtherFormatsDescriptor;

/**
 * This is a simple example of a PicApport plugin. 
 * To compile this at least a picapport.jar version 3.3 or newer is required.
 * This Plugin supports .gif and .png files.
 * 
 * PicApport plugins for other fileformats must implement de.contecon.picapport.plugins.otherformats.IOtherFileFormat
 * 
 * Please have in mind that a valid PicApport plugin .jar files MUST have a Manifest.mf file
 * with a PicApport-Plugin: entry.
 * 
 * If a plugin requires other .jar-files than picapport.jar, these files must be declared in the manifest.mf file
 * by using the Class-Path: entry  
 *  
 * This Plugin also uses pdfbox-app-1.8.8.jar released under the Apache 2.0 License
 * 
 * Apache PDFBox application
 * Copyright 2002-2014 The Apache Software Foundation
 * This product includes software developed at
 * The Apache Software Foundation (http://www.apache.org/).
 *
 *  
 * Example Manifest.mf:
 * <pre>
 * Manifest-Version: 1.0
 * Sealed: true
 * PicApport-Plugin: de.contecon.picapport.plugin.pdf.PdfPlugin
 * Class-Path: pdfbox-app-1.8.8.jar
 * <pre>
 * 10.02.2015
 * @author Eric
 */
public class PdfPlugin implements IOtherFileFormat {

  private int resolution = 96;
  
  private IPicApportPlugInLogger logger;

  
   
  
  @Override
  public List<OtherFormatsDescriptor> init(File pluginDirectory, Properties props, IPicApportPlugInLogger logger) {
    this.logger = logger;
    
    if(props != null) {
      setResolutionFromString(props.getProperty("pdf.resolution"));
      }
    
    return  Arrays.asList(
              new OtherFormatsDescriptor[] {
                  new OtherFormatsDescriptor(".pdf", 
                      "application/pdf", 
                      true, 
                      "PicApport PDF plugin", 
                      "(c) 2015 Contecon Software GmbH", 
                      "1.0", 
                      props),
                      
              });   
  }

 

  @Override
  public CcXMPMetaData createJpegFile(File otherFormatFile, File jpegFileToCreate, CcXMPMetaData metaDataIn) throws Exception {
    createImageFromPdf(otherFormatFile, jpegFileToCreate,metaDataIn);
    return metaDataIn;
  }

  
  /**
   * helper method 
   * author ES 30.01.2015
   * @param sRes String to parse
   */
  private void setResolutionFromString(String sRes) {
    int newResolution =resolution;
    
    if(sRes != null) {
      try {
          int i = Integer.parseInt(sRes);
          if(i>0) {
            newResolution=Math.max(72, Math.min(600 ,i));
            }
          }
      catch(Exception e) {
          }
      }
    this.resolution = newResolution;
    logger.LogInfoMessage("PDF-Resolution set to "+this.resolution);
  }
  
  
  /**
   * helper method to create a dummy substitute in case of one of the following conditions
   * - an error occurred while reading the pdf
   * - the pdf is encrypted and a password is required
   * 
   * author ES 30.01.2015
   * 
   * @param target File
   * @param resource FileNem in this .jar File
   * @throws IOException 
   */
  private void copyDummySubstitute(File target, String resourceName) throws Exception {
    
    try(InputStream resourceAsStream = PdfPlugin.class.getClassLoader().getResourceAsStream("de/contecon/picapport/plugin/pdf/images/" + resourceName)) {
        Files.copy(resourceAsStream, target.toPath(), StandardCopyOption.REPLACE_EXISTING);
    } 
    
  }
  
  
  /**
   * 
   * uses Apache PDFBox to convert the first page of a pdf to jpg 
   * 
   * author ES 30.01.2015
   * 
   * @param pdfFile
   * @param jpegFileToCreate
   * @param metaDataIn 
   * @throws Exception
   */
  private void createImageFromPdf(File pdfFile, File jpegFileToCreate, CcXMPMetaData metaDataIn) throws Exception {
     PDDocument document = null;
     try {
          document = PDDocument.load( pdfFile );
          if(document.isEncrypted()) {
            copyDummySubstitute(jpegFileToCreate, "PDF-Encrypted.jpg");
            return;
            }
          String outputName=cutExtension(jpegFileToCreate.getAbsolutePath());
          
          File pageOneFile=new File(outputName+"1.jpg"); // PDFBox always adds the page number to the jpg filename 
          
          PDFImageWriter pdfImageWriter = new PDFImageWriter();
          boolean success = pdfImageWriter.writeImage(document, 
                                                      "jpg",  
                                                      "", // password 
                                                      1,  // startPage
                                                      1,  // endPage
                                                      outputName, 
                                                      BufferedImage.TYPE_INT_RGB,
                                                      resolution);
          if(!success) {
             logger.LogErrorMessage("PDFImageWriter.writeImage was not successful. Dummy substitute created.");
             pageOneFile.delete();
             copyDummySubstitute(jpegFileToCreate, "PDF-Corrupted.jpg");
           }
          
          jpegFileToCreate.delete();
          pageOneFile.renameTo(jpegFileToCreate);
          
          if(metaDataIn != null) {
            parseMetaData(document, metaDataIn);                
            }
         }  
     catch(Exception e) {
         logger.LogErrorMessage("Unecpected Exception in createImageFromPdf: "+e.getLocalizedMessage()); 
         copyDummySubstitute(jpegFileToCreate, "PDF-Corrupted.jpg");
     }
     finally {
       if(document != null) {
         document.close();
       }
     }  
     
  }

  /**
   * helper method remove .jpg from target file
   * 
   * author ES 30.01.2015
   * @param absolutePath
   * @return
   */
  private String cutExtension(String absolutePath) {
    return absolutePath.substring(0, absolutePath.length()-3);
  }
  



  private void parseMetaData(PDDocument document, CcXMPMetaData metaDataIn) throws IOException, XMPException {
    PDDocumentInformation documentInformation = document.getDocumentInformation();
    
    if(documentInformation.getCreationDate() != null) {
      metaDataIn.setCreationDate(documentInformation.getCreationDate());
      }
    
    // Dublin core has no Author field so we add author and creator to dc:creator 
    // pleas note, that dc:creator is currently not used in PicApport
    String author  = documentInformation.getAuthor();
    String creator = documentInformation.getCreator();
    if(creator != null | author != null) {
      StringBuffer sb=new StringBuffer();
      if(creator!=null) {
        sb.append(creator);
        }
      if(author !=null) {
        if(sb.length()>0) {
          sb.append(", ");
          }
        sb.append(author);  
      }  
    metaDataIn.setCreator(sb.toString());
    }
    
    if(documentInformation.getTitle() != null) {
      metaDataIn.setTitle(documentInformation.getTitle());  
    }
        
    if(documentInformation.getKeywords() != null) {
      String[] split = documentInformation.getKeywords().split("[;, ]");
      for (int i = 0; i < split.length; i++) {
        if(split[i].length() > 0) {
          metaDataIn.addSubject(split[i]);
        }
      }
    }
    
    
    if(documentInformation.getSubject() != null) { // Beschreibung
       metaDataIn.setDescription(documentInformation.getSubject()); 
    }
    
    
  }




  
  

}
